<?php
/*
* GamePHP: Open-source library for game development
* Copyright (C) 2005 Jawad Stouli <jawad.xos@gmail.com>
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
 * Game Engine
 *
 * This file contains the main library class (Engine) and additional includes.
 * This is the ONLY file you must include in your files for using the engine;
 * Then, you can load engine's files this way:
 * <code>
 * require Engine::file("Globals");
 * require Engine::file("utils/Error");
 * require Engine::file("utils/Functions");
 * </code>
 *
 * @package Engine
 * @author Jawad <jawad.xos@gmail.com>
 * @copyright 2005 GamePHP
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 */

/**
 * Main class
 *
 * @package Engine
 * @author Jawad <jawad.xos@gmail.com>
 */
abstract class Engine {
	/**
	 * Classes not allowed to be rewrited.
	 */
	static $not_rewrite = array("Action", "Engine", "Entity", "Globals", "Map", "MapZone", "Object");

	/**
	 * Getting path of an engine's/game's file.
	 *
	 * It's highly recommended to include engine's file with this function.
	 * You can then move engine path without having to change each include statement (just config file).
	 * <code>
	 * require Engine::file("Action");
	 * require Engine::file("utils/Capture");
	 * </code>
	 *
	 * @param String Filename (without extension, relative path)
	 *
	 * @since 0.1
	 *
	 * @return String Complete filename (path + filename + extension)
	 */
    public function file($fName) {
		if(!in_array($fName, self::$not_rewrite) && defined("_GAMEPHP_GAME_DIR") && file_exists(_GAMEPHP_GAME_DIR.$fName.".php")) {
    		return _GAMEPHP_GAME_DIR.$fName.".php";
    	} else {
    		return _GAMEPHP_ENGINE_DIR.$fName.".php";
    	}
    }
    
    /**
	 * Getting path to a gamefile.
	 *
	 * @param String Filename (without extension, relative path)
	 *
	 * @since 0.1
	 *
	 * @return String Complete filename (path + filename + extension)
	 */
    public function gamefile($fName) {
		return _GAMEPHP_GAME_DIR.$fName.".php";
    }
    
    /**
     * Initialiser une entit
     *
     * @param $type String Type d'entit
     * @param $id Mixed Identifiant
     *
     * @return Mixed[False,Object]
     */
    public function &getEntity($type, $id) {
    	if(file_exists(Engine::file("entities/{$type}"))) {
    		require_once Engine::file("entities/{$type}");
    		
    		return new $type($id);
    	} else {
    		return false;	
    	}
    }
    
    /**
 	 * Instancier une carte
 	 *
	 * @param Int $mapid Identifiant unique de la carte dans la table `maps` (interne)
	 *
	 * @return Map
	 */
	public function &getMap($mapid) {
		$mInfos = infoMap($mapid);
			
		if(!file_exists(Engine::file("maps/".$mInfos["type"]))) return Warning(2001, "&getMap", __FILE_, __LINE__, array("Informations sur la carte" => $mInfos));
			
		require_once Engine::file("maps/".$mInfos["type"]);
			
		$map = new $mInfos["type"]($mInfos);
		
		EventManager::addListener("Entity", array("f" => "EntityCallback", "o" => $map));
		
		return $map; 
	}
}

/**
 * Gestionnaire des vnements
 */
require Engine::file("Events");

/**
 * General declarations (configuration, errors...).
 *
 * @see Globals.php
 */
require Engine::file("Globals");

/**
 * Database functions
 */
require_once Engine::file("utils/Database");

/**
 * General functions
 */
require Engine::file("utils/Functions");

/**
 * Returned values
 */
require Engine::file("utils/Result");

/**
 * Inclusion de la classe Action
 */
require Engine::file("Action");

/**
 * Inclusion de l'interface Entity
 */
require Engine::file("Entity");

/**
 * Inclusion de l'interface Map
 */
require Engine::file("Map");

/**
 * Set engine as loaded
 */
define("_GAMEPHP_ENGINE", true);
?>