<?php
/*
* GamePHP: Open-source library for game development
* Copyright (C) 2005 Jawad Stouli <jawad.xos@gmail.com>
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
 * Gestionnaire de cartes
 *
 * @package Engine
 * @author Jawad <jawad.xos@gmail.com>
 */
 
/**
 * Fonctions d'accs  la base de donnes.
 */
require_once Engine::file("utils/SpecModel");

/**
 * On inclut la classe MyIterator
 */
require_once Engine::file("utils/MyIterator");

/**
 * On inclut la classe MapZone
 */
require Engine::file("MapZone");

/**
 * Obtenir des informations sur une carte
 *
 * @param Int $objectid Identifiant unique de l'objet dans la table `objects` (interne)
 * 
 * @return Object
 */
function infoMap($mapid) {
	return getConnection()->get("*", "maps", array("id" => (int)$mapid));
}

/**
 * Interface de base servant  la dfinition d'un nouveau type de carte
 *
 * Cette interface dfinit l'ensemble des fonctions dont une classe l'implmentant (donc un nouveau type de carte) doit disposer afin d'tre exploitable par le moteur.
 *
 * @package Engine
 * @subpackage Maps
 * @author Jawad <jawad.xos@gmail.com>
 */
interface Map {
	/**
	 * Obtenir la liste des entits qui entourent une position
	 *
	 * @param Array $coords Coordonnes de la position
	 * @param Array $ray Nombre de cases autour de la position
	 *
	 * @return Mixed
	 */
	public function getEntitiesAround($coords, $ray);

	/**
	 * Obtenir les calques d'une position
	 *
	 * Le paramtre $zone est optionnel:
	 *   - vide: les informations seront rcupres directement depuis la base de donnes
	 *   - "buffer": les informations seront rcupres depuis la dernire zone cre avec getZone (si possible, sinon depuis la base de donnes)
	 *   - variable de type MapZone: les informations seront rcupres depuis la zone fournie en paramtre
	 *
	 * @param Array $coords Tableau de coordonnes (x, y)
	 * @param Mixed[Void, String, MapZone] $zone Zone d'o les calques doivent tre rcuprs
	 *
	 * @return Array
	 */
	public function getPositionLayers($coords, &$zone = "");
	
	/**
	 * Obtenir le calque d'un type (Entity, Tile, Object...)
	 *
	 * @param String $type Type
	 *
	 * @return Int
	 */
	public function getTypeLayer($type);
	
	/**
	 * Crer une zone
	 *
	 * Cette fonction permet de d'obtenir les informations des positions de la zone depuis la base de donnes.
	 *
	 * @param Array $coords Tableau de coordonnes (x1, x2, y1, y2)
	 *
	 * @return MapZone
	 */
	public function &getZone($coords);
	
	/**
	 * Check if an entity can walk on a case.
	 *
	 * @param Array $coords Array of coordinates (x, y)
	 * @param MapZone $zone Instance of MapZone -not compulsory, depends if you have a zone or not-
	 *
	 * @return Bool
	 */
	public function isWalkable($coords, &$zone = "");
	
	/**
	 * Vrifier qu'une position existe sur la carte ( coordmin <= coord <= coordmax)
	 *
	 * @param Array $coords Tableau de coordonnes
	 *
	 * @return Bool
	 */
	public function positionIn($coords);
	
	/**
	 * Dfinir un calque d'une position dans la base de donnes
	 *
	 * Cette fonction permet de dfinir les calques un par un.
	 *
	 * @param Array $coords Tableau de coordonnes (x, y)
	 * @param String $type Type du calque (Tile, Object, Event...)
	 * @param Int $id Identifiant unique du type associ
	 *
	 * @return Mixed[Error,Bool]
	 */
	public function setPositionLayer($coords, $type, $id);
	
	/**
	 * Set a zone to be used when $zone is empty.
	 *
	 * @param MapZone $zone
	 *
	 * @since 0.1
	 */
	public function useDefaultZone(MapZone &$zone);
}

/**
 * Itrateur de carte
 *
 * Permet d'effectuer une boucle sur tous les couples x/y d'une carte, et de rcuprer les calques associs
 *
 * @package Engine
 * @subpackage Maps
 * @author Jawad <jawad.xos@gmail.com>
 */
class MapIterator extends MyIterator {
	/**
	 * Carte
	 */
	private $map;
	
	/**
	 * Constructeur
	 *
	 * @param Map $map Carte
	 *
	 * @return MapIterator
	 */
	public function __construct(Map $map) {		
		$this->map = $map;
		
		parent::__construct($map->mInfos["coords"]);
	}
	
	/**
	 * Construire le tableau de rsultat
	 *
	 * @return Array
	 */
	protected function buildResult() {
		return array("x" => $this->currx, "y" => $this->curry, "layers" => $this->map->getPositionLayers(array("x" => $this->currx, "y" => $this->curry)));
	}
	
	/**
	 * Obtenir les calques d'une position, et avancer  la position suivante (automatiquement)
	 *
	 * @return Array
	 */
	public function fetchPosition() {
		return parent::fetch();
	}
}

/**
 * Not walkable value
 */
define("_GAMEPHP_MAPS_NOTWALK", "nw");
?>